/*
 * Copyright 2022 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "wBlowingResidual.h"
#include "wF0El.h"

#include "wElement.h"
#include "wCache.h"
#include "wGauss.h"

using namespace tbox;
using namespace flow;

/**
 * @brief Build the residual vector, on one boundary element
 *
 * b = \int psi * vn dV
 */
Eigen::VectorXd BlowingResidual::build(Element const &e, std::vector<double> const &phi, F0El const &f)
{
    // Get pre-computed values
    Cache &cache = e.getVCache();
    Gauss &gauss = cache.getVGauss();

    // Blowing velocity
    double vn = f.eval(e, phi, 0);

    // Integral of (normal) transpiration velocity
    Eigen::VectorXd b = Eigen::VectorXd::Zero(e.nodes.size());
    for (size_t k = 0; k < gauss.getN(); ++k)
        b += cache.getSf(k) * vn * gauss.getW(k) * e.getDetJ(k);
    return b;
}
