/*
 * Copyright 2022 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "wKuttaElement.h"
#include "wNode.h"
using namespace tbox;
using namespace flow;

KuttaElement::KuttaElement(size_t _no, Element *&_surE, Element *&_volE,
                           std::vector<std::pair<size_t, Node *>> &_teN, int _sign)
    : wObject(), no(_no), surE(_surE), volE(_volE),
      teN(_teN), sign(_sign)

{
    // Sanity checks
    if (surE->type() == ELTYPE::LINE2)
        nDim = 2;
    else
    {
        std::stringstream err;
        err << "KuttaElement only implemented for surface elements of type "
            << ELTYPE::LINE2 << " (" << surE->type() << " was given)!\n";
        throw std::runtime_error(err.str());
    }
    if (volE->type() == ELTYPE::TRI3)
        nCol = volE->nodes.size();
    else
    {
        std::stringstream err;
        err << "KuttaElement only implemented for volume elements of type "
            << ELTYPE::TRI3 << " (" << volE->type() << " was given)!\n";
        throw std::runtime_error(err.str());
    }
    // Count TE nodes
    nRow = teN.size();
}

void KuttaElement::write(std::ostream &out) const
{
    out << "KuttaElement #" << no
        << "\n\ton surface element (" << surE->type() << "): " << surE->no
        << "\n\ton volume element: (" << volE->type() << "): " << volE->no
        << "\n\twith " << teN.size() << " TE nodes: ";
    for (auto p : teN)
        out << p.second->no << " ";
    out << "\n\twith sign: " << sign << std::endl;
}