/*
 * Copyright 2022 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WMEDIUM_H
#define WMEDIUM_H

#include "flow.h"
#include "wGroup.h"
#include "wF0El.h"
#include "wF0Ps.h"
#include <map>

namespace flow
{

/**
 * @brief Handle fluid medium
 * @authors Adrien Crovato
 */
class FLOW_API Medium : public tbox::Group
{
private:
    size_t nCnt; ///< number of nodes
public:
    std::map<tbox::Node *, std::vector<tbox::Element *>> neMap; ///< map each elements connected to a node

#ifndef SWIG
    F0El *rho;          ///< density
    F0El *mach;         ///< Mach number
    F0El *cP;           ///< pressure coefficient
    F0PsPhiInf *phiInf; ///< freestream potential
#endif

    std::vector<std::pair<tbox::Element *, std::vector<tbox::Element *>>> adjMap; ///< map of adjacent elements

    Medium(std::shared_ptr<tbox::MshData> _msh, int no, double mInf, int dim, double alpha, double beta);
    Medium(std::shared_ptr<tbox::MshData> _msh, std::string const &name, double mInf, int dim, double alpha, double beta);
    virtual ~Medium();

#ifndef SWIG
    void initFun(double mInf, int dim, double alpha, double beta);
    void createMap();
    virtual void write(std::ostream &out) const override;
#endif
};

} // namespace flow

#endif //WMEDIUM_H
